/*
 * Copyright (c) 2004 - 2010, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package org.tranql.connector.informix;


import java.sql.Connection;
import java.sql.SQLException;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.security.auth.Subject;
import javax.sql.XAConnection;

import org.tranql.connector.CredentialExtractor;
import org.tranql.connector.NoExceptionsAreFatalSorter;
import org.tranql.connector.jdbc.AbstractXADataSourceMCF;
import org.tranql.connector.jdbc.ConnectionWrapper;
import org.tranql.connector.jdbc.ManagedXAConnection;

import com.informix.jdbcx.IfxXADataSource;

/**
 * @version $Revision: 722 $ $Date: 2010-05-10 14:16:37 -0700 (Mon, 10 May 2010) $
 */

public class XAMCF extends AbstractXADataSourceMCF<IfxXADataSource> {
    
    private String password = "";
    private int preparedStatementCacheSize = 0; 

    /**
     * Default constructor for an Informix XA DataSource.
     */
    public XAMCF() {
        super(new IfxXADataSource(), new NoExceptionsAreFatalSorter());
    }
 
    public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo connectionRequestInfo) throws ResourceException {
        CredentialExtractor credentialExtractor = new CredentialExtractor(subject, connectionRequestInfo, this);

        XAConnection sqlConnection = getPhysicalConnection(subject, credentialExtractor);
        try {
            ManagedXAConnection mxac = null;
            if (preparedStatementCacheSize > 0) {
                Connection pc = new ConnectionWrapper(sqlConnection.getConnection(), preparedStatementCacheSize);
                mxac = new ManagedXAConnection(this, sqlConnection, sqlConnection.getXAResource(), pc, credentialExtractor, exceptionSorter);
            } else {
                mxac = new ManagedXAConnection(this, sqlConnection, credentialExtractor, exceptionSorter);
            }
            return mxac;
        } catch (SQLException e) {
            throw new ResourceAdapterInternalException("Could not set up ManagedXAConnection", e);
        }
    }
    
    public void setServerName(String sn) {
        xaDataSource.setServerName(sn);
    }

    public String getServerName() {
        return xaDataSource.getServerName();
    }
    
    public void setDatabaseName(String dbname) {
        xaDataSource.setDatabaseName(dbname);
    }

    public String getDatabaseName() {
        return xaDataSource.getDatabaseName();
    }

    public void setUserName(String user) {
        xaDataSource.setUser(user);
    }

    public String getUserName() {
        return xaDataSource.getUser();
    }
    
    public void setPassword(String pwd) {
        this.password = pwd;
        xaDataSource.setPassword(pwd);
    }

    public String getPassword() {
        return password;
    }

    public void setPortNumber(Integer i) {
        xaDataSource.setPortNumber(nullToZero(i));
    }

    public Integer getPortNumber() {
        return xaDataSource.getPortNumber();
    }
    
    public Integer getPreparedStatementCacheSize() {
        return preparedStatementCacheSize;
    }

    public void setPreparedStatementCacheSize(Integer preparedStatementCacheSize) {
        this.preparedStatementCacheSize = preparedStatementCacheSize;
    }

    //The following are Informix specific values.
    @Deprecated
    public void setIfxHost(String ifxHost){
    	if (ifxHost == null || ifxHost.trim().length() == 0) {
            return;
        }
    	xaDataSource.setIfxIFXHOST(ifxHost);
    }
    
    @Deprecated
    public String getIfxHost(){
    	return xaDataSource.getIfxIFXHOST();
    }
    
    @Deprecated
    public void setIfxProtocolTrace(Integer i) {
        xaDataSource.setIfxPROTOCOLTRACE(nullToZero(i));
    }

    @Deprecated
    public Integer getIfxProtocolTrace() {
        return xaDataSource.getIfxPROTOCOLTRACE();
    }
    
    @Deprecated
    public void setIfxProtocolTraceFile(String ifxPTF) {
        if (ifxPTF == null || ifxPTF.trim().length() == 0) {
            return;
        }
        xaDataSource.setIfxPROTOCOLTRACEFILE(ifxPTF);
    }
   
    @Deprecated
    public String getIfxProtocolTraceFile() {
        return xaDataSource.getIfxPROTOCOLTRACEFILE();
    }
    
    @Deprecated
    public void setIfxSQLHostFile(String ifxSF) {
        if (ifxSF == null || ifxSF.trim().length() == 0) {
            return;
        }
        xaDataSource.setIfxSQLH_FILE(ifxSF);
    }
   
    @Deprecated
    public String getIfxSQLHostFile() {
        return xaDataSource.getIfxSQLH_FILE();
    }
    
    @Deprecated
    public void setIfxSQLHostType(String ifxST) {
    	if (ifxST == null || ifxST.trim().length() == 0) {
            return;
        }
        xaDataSource.setIfxSQLH_TYPE(ifxST);
    }
    
    @Deprecated
    public String getIfxSQLHostType() {
        return xaDataSource.getIfxSQLH_TYPE();
    }

    //USE THESE
    public String getDataSourceName() {
        return xaDataSource.getDataSourceName();
    }

    public void setDataSourceName(String s) {
        xaDataSource.setDataSourceName(s);
    }

    public String getDescription() {
        return xaDataSource.getDescription();
    }

    public void setDescription(String s) {
        xaDataSource.setDescription(s);
    }

    public String getRoleName() {
        return xaDataSource.getRoleName();
    }

    public void setRoleName(String s) {
        xaDataSource.setRoleName(s);
    }

    public void setIfxDirectConnection() {
        xaDataSource.setIfxDirectConnection();
    }

    public void setIfxPORTNO_SECONDARY(Integer i) {
        xaDataSource.setIfxPORTNO_SECONDARY(nullToZero(i));
    }

    public Integer getIfxPORTNO_SECONDARY() {
        return xaDataSource.getIfxPORTNO_SECONDARY();
    }

    public void setIfxIFXHOST_SECONDARY(String s) {
        xaDataSource.setIfxIFXHOST_SECONDARY(s);
    }

    public String getIfxIFXHOST_SECONDARY() {
        return xaDataSource.getIfxIFXHOST_SECONDARY();
    }

    public void setIfxINFORMIXSERVER_SECONDARY(String s) {
        xaDataSource.setIfxINFORMIXSERVER_SECONDARY(s);
    }

    public String getIfxINFORMIXSERVER_SECONDARY() {
        return xaDataSource.getIfxINFORMIXSERVER_SECONDARY();
    }

    public void setIfxENABLE_HDRSWITCH(Boolean b) {
        xaDataSource.setIfxENABLE_HDRSWITCH(b);
    }

    public Boolean getIfxENABLE_HDRSWITCH() {
        return xaDataSource.getIfxENABLE_HDRSWITCH();
    }

    public String getIfxIFXHOST() {
        return xaDataSource.getIfxIFXHOST();
    }

    public void setIfxIFXHOST(String s) {
        xaDataSource.setIfxIFXHOST(s);
    }

    public String getIfxJDBCTEMP() {
        return xaDataSource.getIfxJDBCTEMP();
    }

    public void setIfxJDBCTEMP(String s) {
        xaDataSource.setIfxJDBCTEMP(s);
    }

    public String getIfxSECURITY() {
        return xaDataSource.getIfxSECURITY();
    }

    public String getIfxCSM() {
        return xaDataSource.getIfxCSM();
    }

    public void setIfxSECURITY(String s) {
        xaDataSource.setIfxSECURITY(s);
    }

    public void setIfxCSM(String s) {
        xaDataSource.setIfxCSM(s);
    }

    public String getIfxPROXY() {
        return xaDataSource.getIfxPROXY();
    }

    public void setIfxPROXY(String s) {
        xaDataSource.setIfxPROXY(s);
    }

    public Boolean isIfxENABLE_TYPE_CACHE() {
        return xaDataSource.isIfxENABLE_TYPE_CACHE();
    }

    public void setIfxENABLE_TYPE_CACHE(Boolean b) {
        xaDataSource.setIfxENABLE_TYPE_CACHE(b);
    }

    public String getIfxSQLH_TYPE() {
        return xaDataSource.getIfxSQLH_TYPE();
    }

    public void setIfxSQLH_TYPE(String s) {
        xaDataSource.setIfxSQLH_TYPE(s);
    }

    public String getIfxSQLH_FILE() {
        return xaDataSource.getIfxSQLH_FILE();
    }

    public void setIfxSQLH_FILE(String s) {
        xaDataSource.setIfxSQLH_FILE(s);
    }

    public String getIfxLDAP_URL() {
        return xaDataSource.getIfxLDAP_URL();
    }

    public void setIfxLDAP_URL(String s) {
        xaDataSource.setIfxLDAP_URL(s);
    }

    public String getIfxLDAP_IFXBASE() {
        return xaDataSource.getIfxLDAP_IFXBASE();
    }

    public void setIfxLDAP_IFXBASE(String s) {
        xaDataSource.setIfxLDAP_IFXBASE(s);
    }

    public String getIfxLDAP_USER() {
        return xaDataSource.getIfxLDAP_USER();
    }

    public void setIfxLDAP_USER(String s) {
        xaDataSource.setIfxLDAP_USER(s);
    }

    public String getIfxLDAP_PASSWD() {
        return xaDataSource.getIfxLDAP_PASSWD();
    }

    public void setIfxLDAP_PASSWD(String s) {
        xaDataSource.setIfxLDAP_PASSWD(s);
    }

    public String getIfxSQLH_LOC() {
        return xaDataSource.getIfxSQLH_LOC();
    }

    public void setIfxSQLH_LOC(String s) {
        xaDataSource.setIfxSQLH_LOC(s);
    }

    public Integer getIfxFET_BUF_SIZE() {
        return xaDataSource.getIfxFET_BUF_SIZE();
    }

    public void setIfxFET_BUF_SIZE(Integer i) {
        xaDataSource.setIfxFET_BUF_SIZE(nullToZero(i));
    }

    public Integer getIfxBIG_FET_BUF_SIZE() {
        return xaDataSource.getIfxBIG_FET_BUF_SIZE();
    }

    public void setIfxBIG_FET_BUF_SIZE(Integer i) {
        xaDataSource.setIfxBIG_FET_BUF_SIZE(nullToZero(i));
    }

    public Boolean isIfxUSEV5SERVER() {
        return xaDataSource.isIfxUSEV5SERVER();
    }

    public void setIfxUSEV5SERVER(Boolean b) {
        xaDataSource.setIfxUSEV5SERVER(b);
    }

    public Integer getIfxLOBCACHE() {
        return xaDataSource.getIfxLOBCACHE();
    }

    public void setIfxLOBCACHE(Integer i) {
        xaDataSource.setIfxLOBCACHE(nullToZero(i));
    }

    public Boolean isIfxIFX_USEPUT() {
        return xaDataSource.isIfxIFX_USEPUT();
    }

    public void setIfxIFX_USEPUT(Boolean b) {
        xaDataSource.setIfxIFX_USEPUT(b);
    }

    public Boolean isIfxIFX_AUTOFREE() {
        return xaDataSource.isIfxIFX_AUTOFREE();
    }

    public void setIfxIFX_AUTOFREE(Boolean b) {
        xaDataSource.setIfxIFX_AUTOFREE(b);
    }

    public Boolean isIfxDELIMIDENT() {
        return xaDataSource.isIfxDELIMIDENT();
    }

    public void setIfxDELIMIDENT(Boolean b) {
        xaDataSource.setIfxDELIMIDENT(b);
    }

    public Integer getIfxINFORMIXSTACKSIZE() {
        return xaDataSource.getIfxINFORMIXSTACKSIZE();
    }

    public void setIfxINFORMIXSTACKSIZE(Integer i) {
        xaDataSource.setIfxINFORMIXSTACKSIZE(nullToZero(i));
    }

    public String getIfxDBSPACETEMP() {
        return xaDataSource.getIfxDBSPACETEMP();
    }

    public void setIfxDBSPACETEMP(String s) {
        xaDataSource.setIfxDBSPACETEMP(s);
    }

    public String getIfxDB_LOCALE() {
        return xaDataSource.getIfxDB_LOCALE();
    }

    public void setIfxDB_LOCALE(String s) {
        xaDataSource.setIfxDB_LOCALE(s);
    }

    public String getIfxCLIENT_LOCALE() {
        return xaDataSource.getIfxCLIENT_LOCALE();
    }

    public void setIfxCLIENT_LOCALE(String s) {
        xaDataSource.setIfxCLIENT_LOCALE(s);
    }

    public String getIfxNEWLOCALE() {
        return xaDataSource.getIfxNEWLOCALE();
    }

    public String getIfxNEWNLSMAP() {
        return xaDataSource.getIfxNEWNLSMAP();
    }

    public void setIfxNEWLOCALE(String s) {
        xaDataSource.setIfxNEWLOCALE(s);
    }

    public void setIfxNEWNLSMAP(String s) {
        xaDataSource.setIfxNEWNLSMAP(s);
    }

    public String getIfxNEWCODESET() {
        return xaDataSource.getIfxNEWCODESET();
    }

    public void setIfxNEWCODESET(String s) {
        xaDataSource.setIfxNEWCODESET(s);
    }

    public String getIfxDBDATE() {
        return xaDataSource.getIfxDBDATE();
    }

    public void setIfxDBDATE(String s) {
        xaDataSource.setIfxDBDATE(s);
    }

    public String getIfxGL_DATE() {
        return xaDataSource.getIfxGL_DATE();
    }

    public void setIfxGL_DATE(String s) {
        xaDataSource.setIfxGL_DATE(s);
    }

    public String getIfxDBTIME() {
        return xaDataSource.getIfxDBTIME();
    }

    public void setIfxDBTIME(String s) {
        xaDataSource.setIfxDBTIME(s);
    }

    public String getIfxGL_DATETIME() {
        return xaDataSource.getIfxGL_DATETIME();
    }

    public void setIfxGL_DATETIME(String s) {
        xaDataSource.setIfxGL_DATETIME(s);
    }

    public Boolean isIfxUSE_DTENV() {
        return xaDataSource.isIfxUSE_DTENV();
    }

    public void setIfxUSE_DTENV(Boolean b) {
        xaDataSource.setIfxUSE_DTENV(b);
    }

    public String getIfxDBCENTURY() {
        return xaDataSource.getIfxDBCENTURY();
    }

    public void setIfxDBCENTURY(String s) {
        xaDataSource.setIfxDBCENTURY(s);
    }

    public String getIfxSTMT_CACHE() {
        return xaDataSource.getIfxSTMT_CACHE();
    }

    public void setIfxSTMT_CACHE(String s) {
        xaDataSource.setIfxSTMT_CACHE(s);
    }

    public String getIfxNODEFDAC() {
        return xaDataSource.getIfxNODEFDAC();
    }

    public void setIfxNODEFDAC(String s) {
        xaDataSource.setIfxNODEFDAC(s);
    }

    public String getIfxDBTEMP() {
        return xaDataSource.getIfxDBTEMP();
    }

    public void setIfxDBTEMP(String s) {
        xaDataSource.setIfxDBTEMP(s);
    }

    public String getIfxPSORT_DBTEMP() {
        return xaDataSource.getIfxPSORT_DBTEMP();
    }

    public void setIfxPSORT_DBTEMP(String s) {
        xaDataSource.setIfxPSORT_DBTEMP(s);
    }

    public String getIfxPSORT_NPROCS() {
        return xaDataSource.getIfxPSORT_NPROCS();
    }

    public void setIfxPSORT_NPROCS(String s) {
        xaDataSource.setIfxPSORT_NPROCS(s);
    }

    public String getIfxDBUPSPACE() {
        return xaDataSource.getIfxDBUPSPACE();
    }

    public void setIfxDBUPSPACE(String s) {
        xaDataSource.setIfxDBUPSPACE(s);
    }

    public String getIfxPDQPRIORITY() {
        return xaDataSource.getIfxPDQPRIORITY();
    }

    public void setIfxPDQPRIORITY(String s) {
        xaDataSource.setIfxPDQPRIORITY(s);
    }

    public String getIfxIFX_DIRECTIVES() {
        return xaDataSource.getIfxIFX_DIRECTIVES();
    }

    public void setIfxIFX_DIRECTIVES(String s) {
        xaDataSource.setIfxIFX_DIRECTIVES(s);
    }

    public String getIfxIFX_EXTDIRECTIVES() {
        return xaDataSource.getIfxIFX_EXTDIRECTIVES();
    }

    public void setIfxIFX_EXTDIRECTIVES(String s) {
        xaDataSource.setIfxIFX_EXTDIRECTIVES(s);
    }

    public String getIfxOPTCOMPIND() {
        return xaDataSource.getIfxOPTCOMPIND();
    }

    public void setIfxOPTCOMPIND(String s) {
        xaDataSource.setIfxOPTCOMPIND(s);
    }

    public Integer getIfxINFORMIXCONRETRY() {
        return xaDataSource.getIfxINFORMIXCONRETRY();
    }

    public void setIfxINFORMIXCONRETRY(Integer i) {
        xaDataSource.setIfxINFORMIXCONRETRY(nullToZero(i));
    }

    public Integer getIfxINFORMIXCONTIME() {
        return xaDataSource.getIfxINFORMIXCONTIME();
    }

    public void setIfxINFORMIXCONTIME(Integer i) {
        xaDataSource.setIfxINFORMIXCONTIME(nullToZero(i));
    }

    public String getIfxINFORMIXOPCACHE() {
        return xaDataSource.getIfxINFORMIXOPCACHE();
    }

    public void setIfxINFORMIXOPCACHE(String s) {
        xaDataSource.setIfxINFORMIXOPCACHE(s);
    }

    public String getIfxPLCONFIG() {
        return xaDataSource.getIfxPLCONFIG();
    }

    public void setIfxPLCONFIG(String s) {
        xaDataSource.setIfxPLCONFIG(s);
    }

    public String getIfxPATH() {
        return xaDataSource.getIfxPATH();
    }

    public void setIfxPATH(String s) {
        xaDataSource.setIfxPATH(s);
    }

    public String getIfxPLOAD_LO_PATH() {
        return xaDataSource.getIfxPLOAD_LO_PATH();
    }

    public void setIfxPLOAD_LO_PATH(String s) {
        xaDataSource.setIfxPLOAD_LO_PATH(s);
    }

    public String getIfxOPT_GOAL() {
        return xaDataSource.getIfxOPT_GOAL();
    }

    public void setIfxOPT_GOAL(String s) {
        xaDataSource.setIfxOPT_GOAL(s);
    }

    public String getIfxOPTOFC() {
        return xaDataSource.getIfxOPTOFC();
    }

    public void setIfxOPTOFC(String s) {
        xaDataSource.setIfxOPTOFC(s);
    }

    public Boolean isIfxDBANSIWARN() {
        return xaDataSource.isIfxDBANSIWARN();
    }

    public void setIfxDBANSIWARN(Boolean b) {
        xaDataSource.setIfxDBANSIWARN(b);
    }

    public void setIfxIFX_GET_SMFLOAT_AS_FLOAT(Integer i) {
        xaDataSource.setIfxIFX_GET_SMFLOAT_AS_FLOAT(nullToZero(i));
    }

    public Integer getIfxIFX_GET_SMFLOAT_AS_FLOAT() {
        return xaDataSource.getIfxIFX_GET_SMFLOAT_AS_FLOAT();
    }

    public void setIfxIFX_SET_FLOAT_AS_SMFLOAT(Integer i) {
        xaDataSource.setIfxIFX_SET_FLOAT_AS_SMFLOAT(nullToZero(i));
    }

    public Integer getIfxIFX_SET_FLOAT_AS_SMFLOAT() {
        return xaDataSource.getIfxIFX_SET_FLOAT_AS_SMFLOAT();
    }

    public void setIfxIFX_CODESETLOB(Integer i) {
        xaDataSource.setIfxIFX_CODESETLOB(nullToZero(i));
    }

    public Integer getIfxIFX_CODESETLOB() {
        return xaDataSource.getIfxIFX_CODESETLOB();
    }

    public void setIfxIFX_BATCHUPDATE_PER_SPEC(Integer i) {
        xaDataSource.setIfxIFX_BATCHUPDATE_PER_SPEC(nullToZero(i));
    }

    public Integer getIfxIFX_BATCHUPDATE_PER_SPEC() {
        return xaDataSource.getIfxIFX_BATCHUPDATE_PER_SPEC();
    }

    public void setIfxIFX_LOCK_MODE_WAIT(Integer i) {
        xaDataSource.setIfxIFX_LOCK_MODE_WAIT(nullToZero(i));
    }

    public Integer getIfxIFX_LOCK_MODE_WAIT() {
        return xaDataSource.getIfxIFX_LOCK_MODE_WAIT();
    }

    public void setIfxIFX_ISOLATION_LEVEL(String s) {
        xaDataSource.setIfxIFX_ISOLATION_LEVEL(s);
    }

    public String getIfxIFX_ISOLATION_LEVEL() {
        return xaDataSource.getIfxIFX_ISOLATION_LEVEL();
    }

    public void setIfxIFX_PAD_VARCHAR(String s) {
        xaDataSource.setIfxIFX_PAD_VARCHAR(s);
    }

    public String getIfxIFX_PAD_VARCHAR() {
        return xaDataSource.getIfxIFX_PAD_VARCHAR();
    }

    public Integer getIfxTRACE() {
        return xaDataSource.getIfxTRACE();
    }

    public void setIfxTRACE(Integer i) {
        xaDataSource.setIfxTRACE(nullToZero(i));
    }

    public Integer getIfxPROTOCOLTRACE() {
        return xaDataSource.getIfxPROTOCOLTRACE();
    }

    public void setIfxPROTOCOLTRACE(Integer i) {
        xaDataSource.setIfxPROTOCOLTRACE(nullToZero(i));
    }

    public String getIfxTRACEFILE() {
        return xaDataSource.getIfxTRACEFILE();
    }

    public void setIfxTRACEFILE(String s) {
        xaDataSource.setIfxTRACEFILE(s);
    }

    public String getIfxPROTOCOLTRACEFILE() {
        return xaDataSource.getIfxPROTOCOLTRACEFILE();
    }

    public void setIfxPROTOCOLTRACEFILE(String s) {
        xaDataSource.setIfxPROTOCOLTRACEFILE(s);
    }

    public String getIfxSQLIDEBUG() {
        return xaDataSource.getIfxSQLIDEBUG();
    }

    public void setIfxSQLIDEBUG(String s) {
        xaDataSource.setIfxSQLIDEBUG(s);
    }

    public void setIfxALLOWREGISTEROUTFORINPARAM(String s) {
        xaDataSource.setIfxALLOWREGISTEROUTFORINPARAM(s);
    }

    public String getIfxALLOWREGISTEROUTFORINPARAM() {
        return xaDataSource.getIfxALLOWREGISTEROUTFORINPARAM();
    }

    public Integer getIfxIFX_TRIMTRAILINGSPACES() {
        return xaDataSource.getIfxIFX_TRIMTRAILINGSPACES();
    }

    public void setIfxIFX_TRIMTRAILINGSPACES(Integer i) {
        xaDataSource.setIfxIFX_TRIMTRAILINGSPACES(nullToZero(i));
    }

    public Integer getIfxIFX_FLAT_UCSQ() {
        return xaDataSource.getIfxIFX_FLAT_UCSQ();
    }

    public void setIfxIFX_FLAT_UCSQ(Integer i) {
        xaDataSource.setIfxIFX_FLAT_UCSQ(nullToZero(i));
    }

    public void setIfxIFX_XASPEC(String s) {
        xaDataSource.setIfxIFX_XASPEC(s);
    }

    public String getIfxIFX_XASPEC() {
        return xaDataSource.getIfxIFX_XASPEC();
    }

    public void setIfxIFX_XASTDCOMPLIANCE_XAEND(Integer i) {
        xaDataSource.setIfxIFX_XASTDCOMPLIANCE_XAEND(nullToZero(i));
    }

    public Integer getIfxIFX_XASTDCOMPLIANCE_XAEND() {
        return xaDataSource.getIfxIFX_XASTDCOMPLIANCE_XAEND();
    }

    /**
     * Equality is defined in terms of the serverName, portNumber and databaseName properties
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof XAMCF) {
        	IfxXADataSource other = ((XAMCF) obj).xaDataSource;
            return equals(xaDataSource.getServerName(), other.getServerName()) &&
                    xaDataSource.getPortNumber() == other.getPortNumber() &&
                    equals(xaDataSource.getDatabaseName(), other.getDatabaseName());
        } else {
            return false;
        }
    }

    private static boolean equals(String a, String b) {
        return a == b || a != null && a.equals(b);
    }

    public int hashCode() {
        return hashCode(xaDataSource.getServerName()) ^ xaDataSource.getPortNumber() ^ hashCode(xaDataSource.getDatabaseName());
    }

    private static int hashCode(String s) {
        return s == null ? 0 : s.hashCode();
    }

    public String toString() {
        return "XAMCF[" + getDatabaseName() + ']';
    }

    private int nullToZero(Integer i) {
        return i== null? 0: i;
    }

}
